@@ -7,7 +7,7 @@ from rdt.transformers.base import BaseTransformer
 from rdt.transformers.null import NullTransformer
 
 
-class BooleanTransformer(BaseTransformer):
+class BinaryEncoder(BaseTransformer):
     """Transformer for boolean data.
 
     This transformer replaces boolean values with their integer representation
@@ -16,41 +16,24 @@ class BooleanTransformer(BaseTransformer):
     Null values are replaced using a ``NullTransformer``.
 
     Args:
-        nan (int or None):
-            Replace null values with the given value. If ``None``, do not replace them.
-            Defaults to ``-1``.
-        null_column (bool):
-            Whether to create a new column to indicate which values were null or not.
-            If ``None``, only create a new column when the fit data contains null values.
-            If ``True``, always create the new column whether there are null values or not.
-            If ``False``, do not create the new column.
-            Defaults to ``None``.
+        missing_value_replacement (object):
+            Indicate what to replace the null values with. If the string ``'mode'`` is given,
+            replace them with the most common value.
+            Defaults to ``mode``.
+        model_missing_values (bool):
+            Whether to create a new column to indicate which values were null or not. The column
+            will be created only if there are null values. If ``True``, create the new column if
+            there are null values. If ``False``, do not create the new column even if there
+            are null values. Defaults to ``False``.
     """
 
-    INPUT_TYPE = 'boolean'
-    DETERMINISTIC_TRANSFORM = True
-    DETERMINISTIC_REVERSE = True
-
+    INPUT_SDTYPE = 'boolean'
     null_transformer = None
 
-    def __init__(self, nan=-1, null_column=None):
-        self.nan = nan
-        self.null_column = null_column
-
-    def get_output_types(self):
-        """Return the output types returned by this transformer.
-
-        Returns:
-            dict:
-                Mapping from the transformed column names to the produced data types.
-        """
-        output_types = {
-            'value': 'float',
-        }
-        if self.null_transformer and self.null_transformer.creates_null_column():
-            output_types['is_null'] = 'float'
-
-        return self._add_prefix(output_types)
+    def __init__(self, missing_value_replacement='mode', model_missing_values=False):
+        super().__init__()
+        self._set_missing_value_replacement('mode', missing_value_replacement)
+        self.model_missing_values = model_missing_values
 
     def _fit(self, data):
         """Fit the transformer to the data.
@@ -59,8 +42,13 @@ class BooleanTransformer(BaseTransformer):
             data (pandas.Series):
                 Data to fit to.
         """
-        self.null_transformer = NullTransformer(self.nan, self.null_column, copy=True)
+        self.null_transformer = NullTransformer(
+            self.missing_value_replacement,
+            self.model_missing_values
+        )
         self.null_transformer.fit(data)
+        if self.null_transformer.models_missing_values():
+            self.output_properties['is_null'] = {'sdtype': 'float', 'next_transformer': None}
 
     def _transform(self, data):
         """Transform boolean to float.
@@ -73,7 +61,7 @@ class BooleanTransformer(BaseTransformer):
                 Data to transform.
 
         Returns
-            pandas.DataFrame or pandas.Series
+            np.ndarray
         """
         data = pd.to_numeric(data, errors='coerce')
         return self.null_transformer.transform(data).astype(float)
@@ -92,13 +80,15 @@ class BooleanTransformer(BaseTransformer):
         if not isinstance(data, np.ndarray):
             data = data.to_numpy()
 
-        if self.nan is not None:
-            data = self.null_transformer.reverse_transform(data)
-
+        data = self.null_transformer.reverse_transform(data)
         if isinstance(data, np.ndarray):
             if data.ndim == 2:
                 data = data[:, 0]
 
             data = pd.Series(data)
 
-        return np.round(data).clip(0, 1).astype('boolean').astype('object')
+        isna = data.isna()
+        data = np.round(data).clip(0, 1).astype('boolean').astype('object')
+        data[isna] = np.nan
+
+        return data
