@@ -1,14 +1,17 @@
 """Transformers for categorical data."""
 
+import warnings
+
 import numpy as np
 import pandas as pd
 import psutil
 from scipy.stats import norm
 
+from rdt.errors import TransformerInputError
 from rdt.transformers.base import BaseTransformer
 
 
-class CategoricalTransformer(BaseTransformer):
+class FrequencyEncoder(BaseTransformer):
     """Transformer for categorical data.
 
     This transformer computes a float representative for each one of the categories
@@ -25,25 +28,18 @@ class CategoricalTransformer(BaseTransformer):
     Null values are considered just another category.
 
     Args:
-        fuzzy (bool):
+        add_noise (bool):
             Whether to generate gaussian noise around the class representative of each interval
             or just use the mean for all the replaced values. Defaults to ``False``.
-        clip (bool):
-            If ``True``, clip the values to [0, 1]. Otherwise normalize them using modulo 1.
-            Defaults to ``False``.
     """
 
-    INPUT_TYPE = 'categorical'
-    OUTPUT_TYPES = {'value': 'float'}
-    DETERMINISTIC_REVERSE = True
-    COMPOSITION_IS_IDENTITY = True
-
+    INPUT_SDTYPE = 'categorical'
+    SUPPORTED_SDTYPES = ['categorical', 'boolean']
     mapping = None
     intervals = None
     starts = None
     means = None
     dtype = None
-    _get_category_from_index = None
 
     def __setstate__(self, state):
         """Replace any ``null`` key by the actual ``np.nan`` instance."""
@@ -55,27 +51,9 @@ class CategoricalTransformer(BaseTransformer):
 
         self.__dict__ = state
 
-    def __init__(self, fuzzy=False, clip=False):
-        self.fuzzy = fuzzy
-        self.clip = clip
-
-    def is_transform_deterministic(self):
-        """Return whether the transform is deterministic.
-
-        Returns:
-            bool:
-                Whether or not the transform is deterministic.
-        """
-        return not self.fuzzy
-
-    def is_composition_identity(self):
-        """Return whether composition of transform and reverse transform produces the input data.
-
-        Returns:
-            bool:
-                Whether or not transforming and then reverse transforming returns the input data.
-        """
-        return self.COMPOSITION_IS_IDENTITY and not self.fuzzy
+    def __init__(self, add_noise=False):
+        super().__init__()
+        self.add_noise = add_noise
 
     @staticmethod
     def _get_intervals(data):
@@ -89,7 +67,26 @@ class CategoricalTransformer(BaseTransformer):
             dict:
                 intervals for each categorical value (start, end).
         """
+        data = data.fillna(np.nan)
         frequencies = data.value_counts(dropna=False)
+        augmented_frequencies = frequencies.to_frame()
+        sortable_column_name = f'sortable_{frequencies.name}'
+        column_name = frequencies.name or 0
+        data_with_new_index = data.reset_index(drop=True)
+        data_is_na = data_with_new_index.isna()
+
+        def tie_breaker(element):
+            if pd.isna(element):
+                return data_is_na.loc[data_is_na == 1].index[0]
+
+            return data_with_new_index.loc[data_with_new_index == element].index[0]
+
+        augmented_frequencies[sortable_column_name] = frequencies.index.map(tie_breaker)
+        augmented_frequencies = augmented_frequencies.sort_values(
+            [column_name, sortable_column_name],
+            ascending=[False, True]
+        )
+        sorted_frequencies = augmented_frequencies[column_name]
 
         start = 0
         end = 0
@@ -98,7 +95,7 @@ class CategoricalTransformer(BaseTransformer):
         intervals = {}
         means = []
         starts = []
-        for value, frequency in frequencies.items():
+        for value, frequency in sorted_frequencies.items():
             prob = frequency / elements
             end = start + prob
             mean = start + prob / 2
@@ -119,18 +116,26 @@ class CategoricalTransformer(BaseTransformer):
     def _fit(self, data):
         """Fit the transformer to the data.
 
-        Create the mapping dict to save the label encoding.
-        Finally, compute the intervals for each categorical value.
+        Compute the intervals for each categorical value.
 
         Args:
             data (pandas.Series):
                 Data to fit the transformer to.
         """
-        self.mapping = {}
         self.dtype = data.dtype
-
         self.intervals, self.means, self.starts = self._get_intervals(data)
-        self._get_category_from_index = list(self.means.index).__getitem__
+
+    @staticmethod
+    def _clip_noised_transform(result, start, end):
+        """Clip transformed values.
+
+        Used to ensure the noise added to transformed values doesn't make it
+        go out of the bounds of a given category.
+
+        The upper bound must be slightly lower than ``end``
+        so it doesn't get treated as the next category.
+        """
+        return np.clip(result, start, end - 1e-9)
 
     def _transform_by_category(self, data):
         """Transform the data by iterating over the different categories."""
@@ -138,14 +143,19 @@ class CategoricalTransformer(BaseTransformer):
 
         # loop over categories
         for category, values in self.intervals.items():
-            mean, std = values[2:]
+            start, end, mean, std = values
             if category is np.nan:
                 mask = data.isna()
             else:
                 mask = (data.to_numpy() == category)
 
-            if self.fuzzy:
-                result[mask] = norm.rvs(mean, std, size=mask.sum())
+            if self.add_noise:
+                result[mask] = norm.rvs(
+                    mean, std,
+                    size=mask.sum(),
+                    random_state=self.random_states['transform']
+                )
+                result[mask] = self._clip_noised_transform(result[mask], start, end)
             else:
                 result[mask] = mean
 
@@ -156,10 +166,11 @@ class CategoricalTransformer(BaseTransformer):
         if pd.isna(category):
             category = np.nan
 
-        mean, std = self.intervals[category][2:]
+        start, end, mean, std = self.intervals[category]
 
-        if self.fuzzy:
-            return norm.rvs(mean, std)
+        if self.add_noise:
+            result = norm.rvs(mean, std, random_state=self.random_states['transform'])
+            return self._clip_noised_transform(result, start, end)
 
         return mean
 
@@ -168,44 +179,46 @@ class CategoricalTransformer(BaseTransformer):
         return data.fillna(np.nan).apply(self._get_value).to_numpy()
 
     def _transform(self, data):
-        """Transform categorical values to float values.
-
-        Replace the categories with their float representative value.
+        """Transform the categorical values to float representatives.
 
         Args:
             data (pandas.Series):
                 Data to transform.
 
         Returns:
-            numpy.ndarray:
+            numpy.ndarray
         """
+        fit_categories = pd.Series(self.intervals.keys())
+        has_nan = pd.isna(fit_categories).any()
+        unseen_indexes = ~(data.isin(fit_categories) | (pd.isna(data) & has_nan))
+        if unseen_indexes.any():
+            # Select only the first 5 unseen categories to avoid flooding the console.
+            unseen_categories = set(data[unseen_indexes][:5])
+            warnings.warn(
+                f'The data contains {unseen_indexes.sum()} new categories that were not '
+                f'seen in the original data (examples: {unseen_categories}). Assigning '
+                'them random values. If you want to model new categories, '
+                'please fit the transformer again with the new data.'
+            )
+
+        data[unseen_indexes] = np.random.choice(fit_categories, size=unseen_indexes.size)
         if len(self.means) < len(data):
             return self._transform_by_category(data)
 
         return self._transform_by_row(data)
 
-    def _normalize(self, data):
-        """Normalize data to the range [0, 1].
-
-        This is done by either clipping or computing the values modulo 1.
-        """
-        if self.clip:
-            return data.clip(0, 1)
-
-        return data % 1
-
     def _reverse_transform_by_matrix(self, data):
         """Reverse transform the data with matrix operations."""
         num_rows = len(data)
-        num_categories = len(self.means)
+        num_categories = len(self.starts)
 
         data = np.broadcast_to(data, (num_categories, num_rows)).T
-        means = np.broadcast_to(self.means, (num_rows, num_categories))
-        diffs = np.abs(data - means)
-        indexes = np.argmin(diffs, axis=1)
+        starts = np.broadcast_to(self.starts.index, (num_rows, num_categories))
+        is_data_greater_than_starts = (data >= starts)[:, ::-1]
+        interval_indexes = num_categories - np.argmax(is_data_greater_than_starts, axis=1) - 1
 
-        self._get_category_from_index = list(self.means.index).__getitem__
-        return pd.Series(indexes).apply(self._get_category_from_index).astype(self.dtype)
+        get_category_from_index = list(self.starts.category).__getitem__
+        return pd.Series(interval_indexes).apply(get_category_from_index).astype(self.dtype)
 
     def _reverse_transform_by_category(self, data):
         """Reverse transform the data by iterating over all the categories."""
@@ -237,8 +250,7 @@ class CategoricalTransformer(BaseTransformer):
         Returns:
             pandas.Series
         """
-        data = self._normalize(data)
-
+        data = data.clip(0, 1)
         num_rows = len(data)
         num_categories = len(self.means)
 
@@ -255,35 +267,7 @@ class CategoricalTransformer(BaseTransformer):
         return self._reverse_transform_by_row(data)
 
 
-class CategoricalFuzzyTransformer(CategoricalTransformer):
-    """Transformer for categorical data.
-
-    This transformer computes a float representative for each one of the categories
-    found in the fit data. Then, when transforming, it replaces the instances of these
-    categories with the corresponding representatives plus some added gaussian noise.
-
-    The representatives are decided by sorting the categorical values by their relative
-    frequency, then dividing the ``[0, 1]`` interval by these relative frequencies, and
-    finally assigning the middle point of each interval to the corresponding category.
-
-    When the transformation is reverted, each value is assigned the category that
-    corresponds to the interval it falls in.
-
-    Null values are considered just another category.
-
-    This class behaves exactly as the ``CategoricalTransformer`` with ``fuzzy=True``.
-
-    Args:
-        clip (bool):
-            If ``True``, clip the values to [0, 1]. Otherwise normalize them using modulo 1.
-            Defaults to ``False``.
-    """
-
-    def __init__(self, clip=False):
-        super().__init__(fuzzy=True, clip=clip)
-
-
-class OneHotEncodingTransformer(BaseTransformer):
+class OneHotEncoder(BaseTransformer):
     """OneHotEncoding for categorical data.
 
     This transformer replaces a single vector with N unique categories in it
@@ -291,18 +275,10 @@ class OneHotEncodingTransformer(BaseTransformer):
     is found and 0s on the rest.
 
     Null values are considered just another category.
-
-    Args:
-        error_on_unknown (bool):
-            If a value that was not seen during the fit stage is passed to
-            transform, then an error will be raised if this is True.
-            Defaults to ``True``.
     """
 
-    INPUT_TYPE = 'categorical'
-    DETERMINISTIC_TRANSFORM = True
-    DETERMINISTIC_REVERSE = True
-
+    INPUT_SDTYPE = 'categorical'
+    SUPPORTED_SDTYPES = ['categorical', 'boolean']
     dummies = None
     _dummy_na = None
     _num_dummies = None
@@ -310,9 +286,6 @@ class OneHotEncodingTransformer(BaseTransformer):
     _indexer = None
     _uniques = None
 
-    def __init__(self, error_on_unknown=True):
-        self.error_on_unknown = error_on_unknown
-
     @staticmethod
     def _prepare_data(data):
         """Transform data to appropriate format.
@@ -340,17 +313,6 @@ class OneHotEncodingTransformer(BaseTransformer):
 
         return data
 
-    def get_output_types(self):
-        """Return the output types produced by this transformer.
-
-        Returns:
-            dict:
-                Mapping from the transformed column names to the produced data types.
-        """
-        output_types = {f'value{i}': 'float' for i in range(len(self.dummies))}
-
-        return self._add_prefix(output_types)
-
     def _fit(self, data):
         """Fit the transformer to the data.
 
@@ -362,7 +324,7 @@ class OneHotEncodingTransformer(BaseTransformer):
         """
         data = self._prepare_data(data)
 
-        null = pd.isna(data)
+        null = pd.isna(data).to_numpy()
         self._uniques = list(pd.unique(data[~null]))
         self._dummy_na = null.any()
         self._num_dummies = len(self._uniques)
@@ -375,6 +337,11 @@ class OneHotEncodingTransformer(BaseTransformer):
         if self._dummy_na:
             self.dummies.append(np.nan)
 
+        self.output_properties = {
+            f'value{i}': {'sdtype': 'float', 'next_transformer': None}
+            for i in range(len(self.dummies))
+        }
+
     def _transform_helper(self, data):
         if self._dummy_encoded:
             coder = self._indexer
@@ -403,18 +370,22 @@ class OneHotEncodingTransformer(BaseTransformer):
                 Data to transform.
 
         Returns:
-            numpy.ndarray:
+            numpy.ndarray
         """
         data = self._prepare_data(data)
-        array = self._transform_helper(data)
+        unique_data = {np.nan if pd.isna(x) else x for x in pd.unique(data)}
+        unseen_categories = unique_data - set(self.dummies)
+        if unseen_categories:
+            # Select only the first 5 unseen categories to avoid flooding the console.
+            examples_unseen_categories = set(list(unseen_categories)[:5])
+            warnings.warn(
+                f'The data contains {len(unseen_categories)} new categories that were not '
+                f'seen in the original data (examples: {examples_unseen_categories}). Creating '
+                'a vector of all 0s. If you want to model new categories, '
+                'please fit the transformer again with the new data.'
+            )
 
-        if self.error_on_unknown:
-            unknown = array.sum(axis=1) == 0
-            if unknown.any():
-                raise ValueError(f'Attempted to transform {list(data[unknown])} ',
-                                 'that were not seen during fit stage.')
-
-        return array
+        return self._transform_helper(data)
 
     def _reverse_transform(self, data):
         """Convert float values back to the original categorical values.
@@ -437,7 +408,7 @@ class OneHotEncodingTransformer(BaseTransformer):
         return pd.Series(indices).map(self.dummies.__getitem__)
 
 
-class LabelEncodingTransformer(BaseTransformer):
+class LabelEncoder(BaseTransformer):
     """LabelEncoding for categorical data.
 
     This transformer generates a unique integer representation for each category
@@ -451,29 +422,70 @@ class LabelEncodingTransformer(BaseTransformer):
         categories_to_values (dict):
             Dictionary that maps each category with the corresponding
             integer value.
+
+    Args:
+        add_noise (bool):
+            Whether to generate uniform noise around the label for each category.
+            Defaults to ``False``.
+        order_by (None or str):
+            A string defining how to order the categories before assigning them labels. Defaults to
+            ``None``. Options include:
+            - ``'numerical_value'``: Order the categories by numerical value.
+            - ``'alphabetical'``: Order the categories alphabetically.
+            - ``None``: Use the order that the categories appear in when fitting.
     """
 
-    INPUT_TYPE = 'categorical'
-    OUTPUT_TYPES = {'value': 'integer'}
-    DETERMINISTIC_TRANSFORM = True
-    DETERMINISTIC_REVERSE = True
-    COMPOSITION_IS_IDENTITY = True
-
+    INPUT_SDTYPE = 'categorical'
+    SUPPORTED_SDTYPES = ['categorical', 'boolean']
     values_to_categories = None
     categories_to_values = None
 
+    def __init__(self, add_noise=False, order_by=None):
+        super().__init__()
+        self.add_noise = add_noise
+        if order_by not in [None, 'alphabetical', 'numerical_value']:
+            raise TransformerInputError(
+                "order_by must be one of the following values: None, 'numerical_value' or "
+                "'alphabetical'"
+            )
+
+        self.order_by = order_by
+
+    def _order_categories(self, unique_data):
+        if self.order_by == 'alphabetical':
+            if unique_data.dtype.type not in [np.str_, np.object_]:
+                raise TransformerInputError(
+                    "The data must be of type string if order_by is 'alphabetical'."
+                )
+
+        elif self.order_by == 'numerical_value':
+            if not np.issubdtype(unique_data.dtype.type, np.number):
+                raise TransformerInputError(
+                    "The data must be numerical if order_by is 'numerical_value'."
+                )
+
+        if self.order_by is not None:
+            nans = pd.isna(unique_data)
+            unique_data = np.sort(unique_data[~nans])  # pylint: disable=E1130
+            if nans.any():
+                unique_data = np.append(unique_data, [np.nan])
+
+        return unique_data
+
     def _fit(self, data):
         """Fit the transformer to the data.
 
         Generate a unique integer representation for each category and
-        store them in the `categories_to_values` dict and its reverse
-        `values_to_categories`.
+        store them in the ``categories_to_values`` dict and its reverse
+        ``values_to_categories``.
 
         Args:
             data (pandas.Series):
                 Data to fit the transformer to.
         """
-        self.values_to_categories = dict(enumerate(pd.unique(data)))
+        unique_data = pd.unique(data.fillna(np.nan))
+        unique_data = self._order_categories(unique_data)
+        self.values_to_categories = dict(enumerate(unique_data))
         self.categories_to_values = {
             category: value
             for value, category in self.values_to_categories.items()
@@ -482,14 +494,39 @@ class LabelEncodingTransformer(BaseTransformer):
     def _transform(self, data):
         """Replace each category with its corresponding integer value.
 
+        If a category has not been seen before, a random value is assigned.
+
+        If ``add_noise`` is True, the integer values will be replaced by a
+        random number between the value and the value + 1.
+
         Args:
             data (pandas.Series):
                 Data to transform.
 
         Returns:
-            numpy.ndarray:
+            pd.Series
         """
-        return pd.Series(data).map(self.categories_to_values)
+        mapped = data.fillna(np.nan).map(self.categories_to_values)
+        is_null = mapped.isna()
+        if is_null.any():
+            # Select only the first 5 unseen categories to avoid flooding the console.
+            unseen_categories = set(data[is_null][:5])
+            warnings.warn(
+                f'The data contains {is_null.sum()} new categories that were not '
+                f'seen in the original data (examples: {unseen_categories}). Assigning '
+                'them random values. If you want to model new categories, '
+                'please fit the transformer again with the new data.'
+            )
+
+        mapped[is_null] = np.random.randint(
+            len(self.categories_to_values),
+            size=is_null.sum()
+        )
+
+        if self.add_noise:
+            mapped = np.random.uniform(mapped, mapped + 1)
+
+        return mapped
 
     def _reverse_transform(self, data):
         """Convert float values back to the original categorical values.
@@ -501,5 +538,71 @@ class LabelEncodingTransformer(BaseTransformer):
         Returns:
             pandas.Series
         """
+        if self.add_noise:
+            data = np.floor(data)
+
         data = data.clip(min(self.values_to_categories), max(self.values_to_categories))
         return data.round().map(self.values_to_categories)
+
+
+class CustomLabelEncoder(LabelEncoder):
+    """Custom label encoder for categorical data.
+
+    This class works very similarly to the ``LabelEncoder``, except that it requires the ordering
+    for the labels to be provided.
+
+    Null values are considered just another category.
+
+    Args:
+        order (list):
+            A list of all the unique categories for the data. The order of the list determines the
+            label that each category will get.
+        add_noise (bool):
+            Whether to generate uniform noise around the label for each category.
+            Defaults to ``False``.
+    """
+
+    def __init__(self, order, add_noise=False):
+        self.order = pd.Series(order).fillna(np.nan)
+        super().__init__(add_noise=add_noise)
+
+    def __repr__(self):
+        """Represent initialization of transformer as text.
+
+        Returns:
+            str:
+                The name of the transformer followed by any non-default parameters.
+        """
+        class_name = self.__class__.get_name()
+        custom_args = []
+        custom_args.append('order=<CUSTOM>')
+        if self.add_noise:
+            custom_args.append(f'add_noise={self.add_noise}')
+
+        args_string = ', '.join(custom_args)
+        return f'{class_name}({args_string})'
+
+    def _fit(self, data):
+        """Fit the transformer to the data.
+
+        Generate a unique integer representation for each category and
+        store them in the ``categories_to_values`` dict and its reverse
+        ``values_to_categories``.
+
+        Args:
+            data (pandas.Series):
+                Data to fit the transformer to.
+        """
+        data = data.fillna(np.nan)
+        missing = list(data[~data.isin(self.order)].unique())
+        if len(missing) > 0:
+            raise TransformerInputError(
+                f"Unknown categories '{missing}'. All possible categories must be defined in the "
+                "'order' parameter."
+            )
+
+        self.values_to_categories = dict(enumerate(self.order))
+        self.categories_to_values = {
+            category: value
+            for value, category in self.values_to_categories.items()
+        }
